/*++
/* NAME
/*	convert_size 3h
/* SUMMARY
/*	string to size conversion
/* SYNOPSIS
/*	#include <convert_size.h>
/* DESCRIPTION
/* .nf

 /*
  * External interface.
  */
extern size_t convert_size(const char *);

/* LICENSE
/* .ad
/* .fi
/*	The IBM Public License must be distributed with this software.
/* AUTHOR(S)
/*	Wietse Venema
/*	IBM T.J. Watson Research
/*	P.O. Box 704
/*	Yorktown Heights, NY 10598, USA
/*--*/
